@@ -17,7 +17,7 @@
  * HostCache.cxx
  *
  *  Created on: Mar 11, 2017
- *      Author: steven
+ *      Author: Steven Hessing (steven.hessing@gmail.com)
  */
 
 #include <iostream>
@@ -26,9 +26,10 @@
 #include <sstream>
 #include <fstream>
 #include <iterator>
-
+#include <vector>
 #include <cstring>
 #include <memory>
+#include <thread>
 
 #include <sys/types.h>
 #include <unistd.h>
@@ -63,17 +64,22 @@ uint32_t HostCache::Prune (bool Force) {
 	    syslog (LOG_DEBUG, "HostCache: starting prune");
 	}
 	uint32_t prunedhosts = 0;
-	for (auto it : hC) {
-        std::string mac = it.second->MacAddress_get();
-        std::string uuid = it.second->Uuid_get();
-		if (it.second->Prune(Force)) {
+
+	for (auto it = hC.begin(); it != hC.end();) {
+        std::string mac = it->second->getMacAddress();
+        std::string uuid = it->second->getUuid();
+		it->second->Prune(Force);
+		if (it->second->isExpired()) {
 			if(uuid != "") {
 			    auto it = dpMap.find(uuid);
 			    if (it != dpMap.end()) {
 			        it->second->removeHost(mac);
 			    }
 			}
+			it = hC.erase (it);
 		    prunedhosts++;
+		} else {
+		    it++;
 		}
 	}
 	if (Debug == true) {
@@ -139,6 +145,9 @@ std::shared_ptr<Host> HostCache::FindOrCreateHostByIp (const std::string inIp, c
 		syslog(LOG_DEBUG, "HostCache: Find or create host for IP %s", inIp.c_str());
 	}
 
+	if (inIp == "0.0.0.0" || inIp == "") {
+	    syslog (LOG_NOTICE, "HostCache: finding host by IP address %s is not possible", inIp.c_str());
+	}
 	if (inIp == "" || WhitelistedNodes.find(inIp) != WhitelistedNodes.end()) {
 		return nullptr;
     }
@@ -147,9 +156,7 @@ std::shared_ptr<Host> HostCache::FindOrCreateHostByIp (const std::string inIp, c
 	if ( it == Ip2MacMap.end()) {
 		Mac = MacLookup(inIp);
 		if (Mac.isValid() == false) {
-			if (Debug == true) {
-				syslog(LOG_DEBUG, "HostCache: Couldn't find ARP entry for %s", inIp.c_str());
-			}
+			syslog(LOG_NOTICE, "HostCache: Got invalid ARP entry %s for %s", Mac.c_str(), inIp.c_str());
 			return nullptr;
 		}
 		Ip2MacMap[inIp] = Mac.get();
@@ -178,7 +185,7 @@ std::shared_ptr<Host> HostCache::FindOrCreateHostByMac (const MacAddress inMac,
 		return nullptr;
     }
 	if (inMac.isValid() == false) {
-		syslog(LOG_WARNING, "HostCache: Empty Mac Address provided");
+		syslog(LOG_WARNING, "HostCache: Invalid Mac Address provided: %s", inMac.c_str());
 		return nullptr;
 	}
 	if (hC.find(inMac.get()) == hC.end()) {
@@ -186,13 +193,19 @@ std::shared_ptr<Host> HostCache::FindOrCreateHostByMac (const MacAddress inMac,
 			syslog(LOG_DEBUG, "HostCache: Adding new Host with MAC address %s for IP %s", inMac.c_str(), inIp.c_str());
 		}
 		auto h = std::make_shared<Host>(inMac.str(), Uuid, Debug);
-		h->IpAddress_set (inIp);
+		h->setIpAddress (inIp);
 		hC[inMac.get()] = h;
 		return h;
 	}
 	if (Debug == true) {
 		syslog(LOG_DEBUG, "HostCache: Found MAC address %s for IP %s", inMac.c_str(), inIp.c_str());
 	}
+	std::shared_ptr<Host> h = hC[inMac.get()];
+	if (inIp != "" && inIp != "0.0.0.0") {
+	     if (h->getIpv4Address() == "0.0.0.0") {
+	         h->setIpAddress(inIp);
+	     }
+	}
 	return hC[inMac.get()];
 }
 
@@ -204,7 +217,7 @@ bool HostCache::AddByMac (const MacAddress inMacAddress, const std::string inIpA
 		return false;
     }
 	auto h = std::make_shared<Host>(inMacAddress, Debug);
-	h->IpAddress_set (inIpAddress);
+	h->setIpAddress (inIpAddress);
 	hC[inMacAddress.get()] = h;
 	Ip2MacMap[inIpAddress] = inMacAddress.get();
 	return true;
@@ -223,11 +236,13 @@ bool HostCache::AddFlow (const std::string srcip, const uint16_t srcport, const
 		return false;
 	}
 
-	std::shared_ptr<Host> h = FindOrCreateHostByIp(srcip);
-	if (h) {
-		h->FlowEntry_set(srcport, dstip, dstport, protocol, FlowExpiration);
-		return true;
-	}
+	try {
+	    std::shared_ptr<Host> h = FindOrCreateHostByIp(srcip);
+	    if (h != nullptr) {
+	        h->setFlowEntry(srcport, dstip, dstport, protocol, FlowExpiration);
+	        return true;
+	    }
+	} catch (...) {}
 	return false;
 }
 
@@ -240,11 +255,13 @@ bool HostCache::AddDnsQueryIp (const std::string clientip, const std::string fqd
 		return false;
 	}
 
-	std::shared_ptr<Host> h = FindOrCreateHostByIp(clientip);
-	if (h) {
-		h->addorupdateDnsQueryList(fqdn);
-		return true;
-	}
+	try {
+	    std::shared_ptr<Host> h = FindOrCreateHostByIp(clientip);
+	    if (h != nullptr) {
+	        h->addorupdateDnsQueryList(fqdn);
+	        return true;
+	    }
+	} catch (...) {}
 	return false;
 }
 
@@ -264,13 +281,21 @@ bool HostCache::AddDhcpRequest (const std::string IpAddress, const MacAddress in
 
 	std::shared_ptr<Host> h;
 	if (inMac.isValid() == true) {
-		h = FindOrCreateHostByMac(inMac, "", IpAddress);
+		try {
+		    h = FindOrCreateHostByMac(inMac, "", IpAddress);
+		} catch (...) {
+		    return false;
+		}
 	} else {
-		h = FindOrCreateHostByIp(IpAddress);
+	    try {
+	        h = FindOrCreateHostByIp(IpAddress);
+	    } catch (...) {
+	        return false;
+        }
 	}
 
-	if (h) {
-		h->Dhcp_set(IpAddress, inMac.str(), Hostname, DhcpVendor);
+	if (h != nullptr) {
+		h->setDhcp(IpAddress, inMac.str(), Hostname, DhcpVendor);
 		return true;
 	}
 	return false;
@@ -289,14 +314,63 @@ bool HostCache::AddSsdpInfo (const std::shared_ptr<SsdpHost> sHost) {
 		return false;
 	}
 
-	std::shared_ptr<Host> h = FindOrCreateHostByIp(sHost->IpAddress);
-	if (h) {
-		h->SsdpInfo_set(sHost);
-		return true;
-	}
+	try {
+	    std::shared_ptr<Host> h = FindOrCreateHostByIp(sHost->IpAddress);
+	    if (h != nullptr) {
+	        h->setSsdpInfo(sHost);
+	        return true;
+	    }
+	} catch (...) {}
 	return false;
 }
 
+bool HostCache::AddWsDiscoveryInfo (const std::shared_ptr<WsDiscoveryHost> inwsdHost) {
+    if (Debug == true) {
+        syslog(LOG_DEBUG, "HostCache: Adding WsDiscovery info for host with IP %s", inwsdHost->IpAddress.c_str());
+    }
+    if (inwsdHost->IpAddress == "") {
+        syslog(LOG_WARNING, "HostCache: AddWsDiscoveryInfo: no IP address provided");
+        return false;
+    }
+    if (WhitelistedNodes.find(inwsdHost->IpAddress) != WhitelistedNodes.end()) {
+        return false;
+    }
+
+    try {
+        std::shared_ptr<Host> h = FindOrCreateHostByIp(inwsdHost->IpAddress);
+        if (h != nullptr) {
+            h->setWsDiscoveryInfo(inwsdHost);
+            return true;
+        }
+    } catch (...) {}
+    return false;
+
+}
+
+bool HostCache::AddMdnsInfo (const std::shared_ptr<MdnsHost> inmdnsHost) {
+    if (Debug == true) {
+        syslog(LOG_DEBUG, "HostCache: Adding mDNS info for host with IP %s", inmdnsHost->IpAddress.c_str());
+    }
+    if (inmdnsHost->IpAddress == "") {
+        syslog(LOG_WARNING, "HostCache: AddMdnsInfo: no IP address provided");
+        return false;
+    }
+    if (WhitelistedNodes.find(inmdnsHost->IpAddress) != WhitelistedNodes.end()) {
+        return false;
+    }
+
+    try {
+        std::shared_ptr<Host> h = FindOrCreateHostByIp(inmdnsHost->IpAddress);
+        if (h != nullptr) {
+            h->setMdnsInfo(inmdnsHost);
+            return true;
+        }
+    } catch (...) {}
+    return false;
+
+}
+
+
 // These functions are for DnsQueryCache
 void HostCache::addorupdateDnsQueryCache (uint16_t id) {
 	DnsQueryCache[id] = time(nullptr) + 15;
@@ -331,9 +405,36 @@ uint32_t HostCache::pruneDnsQueryCache (bool Force) {
 	return deletecount;
 }
 
-// TODO: Lookup MAC addresses in ARP table using IOCTL now works but you need to specify the Ethernet interface and we don't have code for that yet
-// so for now we'll continue doing lookups in /proc/net/arp even though that is slower
-MacAddress HostCache::MacLookup (const std::string inIpAddress, const std::string inInterface, const int retries) {
+MacAddress HostCache::MacLookup (const std::string inIpAddress) {
+    MacAddress Mac("00:00:00:00:00:00");
+    if (LocalIpAddresses.find(inIpAddress) != LocalIpAddresses.end()) {
+        if (Debug == true) {
+            syslog (LOG_DEBUG, "HostCache: Skipping MacLookup of local IP address %s", inIpAddress.c_str());
+        }
+        return Mac;
+    }
+    if (Debug == true) {
+        syslog (LOG_DEBUG, "HostCache: MacLookup of %s", inIpAddress.c_str());
+    }
+    for (auto lanInterface: ifMap->getLanInterfaces()) {
+        Mac = MacLookup(inIpAddress, lanInterface);
+        if (Mac.isValid() == true) {
+            if (Debug == true) {
+                syslog (LOG_DEBUG, "HostCache: Found MAC entry %s on interface %s", Mac.c_str(), lanInterface.c_str());
+            }
+            return Mac;
+        }
+        if (Debug == true) {
+            syslog (LOG_DEBUG, "HostCache: MAC entry not found on interface %s", lanInterface.c_str());
+        }
+    }
+    if (Debug == true) {
+        syslog (LOG_DEBUG, "HostCache: MAC entry not found on lan interfaces");
+    }
+    return Mac;
+}
+
+MacAddress HostCache::MacLookup (const std::string inIpAddress, const std::string inInterface) {
 	int domain;
 	struct arpreq areq;
 	memset(&areq, 0, sizeof(areq));
@@ -361,20 +462,14 @@ MacAddress HostCache::MacLookup (const std::string inIpAddress, const std::strin
     /* Get an internet domain socket. */
     int s;
 	if ((s = socket(domain, SOCK_DGRAM, 0)) == -1) {
-        syslog(LOG_ERR, "HostCache: Can't open socket for ARP table lookup");
+        syslog(LOG_ERR, "HostCache: Can't open socket for ARP table lookup for IP %s, domain %d on interface %s",
+			inIpAddress.c_str(), domain, inInterface.c_str());
         return Mac;
     }
 
     if (-1 == ioctl(s,SIOCGARP , (caddr_t) &areq)) {
-		syslog (LOG_ERR, "HostCache: ARP lookup failure for %s", inIpAddress.c_str());
-		if (retries > 0) {
-			if (Debug == true) {
-				syslog(LOG_DEBUG, "HostCache: Additional ARP lookup for %s", inIpAddress.c_str());
-			}
-			if (SendUdpPing (inIpAddress, 1900)) {
-				usleep(5000);
-				return MacLookup (inIpAddress, inInterface, retries - 1);
-			}
+		if (Debug == true) {
+		    syslog (LOG_DEBUG, "HostCache: ARP lookup failure for %s on interface %s", inIpAddress.c_str(), inInterface.c_str());
 		}
 		return Mac;
 	}
@@ -387,6 +482,7 @@ MacAddress HostCache::MacLookup (const std::string inIpAddress, const std::strin
 	return Mac;
 }
 
+// TODO: We should consolidate this in the IfMap data structure
 uint32_t HostCache::getInterfaceIpAddresses() {
     if (Debug == true) {
         syslog (LOG_DEBUG, "HostCache: discovering IP addresses of network interfaces");
@@ -441,38 +537,6 @@ uint32_t HostCache::getInterfaceIpAddresses() {
 	return 0;
 }
 
-MacAddress HostCache::MacLookup (const std::string inIpAddress, const int retries) {
-	// TODO: we should cache ARP table and only refresh it if a MAC lookup fails
-	if (Debug == true) {
-	    syslog (LOG_DEBUG, "HostCache: deprecated MacLookup of %s", inIpAddress.c_str());
-	}
-	std::ifstream ifs("/proc/net/arp");
-	std::string line;
-	MacAddress Mac;
-	while (std::getline(ifs, line)) {
-		std::smatch m;
-		if(std::regex_match(line, m, arp_rx)) {
-			std::string ip = m.str(1);
-			Mac.set(m.str(2));
-			if (ip == inIpAddress && Mac.isValid() != 0) {
-				ifs.close();
-				return Mac;
-			}
-		}
-
-	}
-	ifs.close();
-	if (retries > 0) {
-		if (Debug == true) {
-			syslog(LOG_DEBUG, "HostCache: Additional ARP lookup for %s", inIpAddress.c_str());
-		}
-		if (SendUdpPing (inIpAddress, 1900)) {
-			usleep(5000);
-			return MacLookup (inIpAddress, retries - 1);
-		}
-	}
-	return Mac;
-}
 
 bool HostCache::SendUdpPing (const std::string DstIpAddress, const uint16_t DstPort) {
 	//Structure for address of server
@@ -529,7 +593,20 @@ bool HostCache::ExportDeviceProfileMatches(const std::string filename, bool deta
 	return true;
 }
 
-uint32_t HostCache::RestApiCall (const std::string api, const json &j, const std::string ClientApiCertFile, const std::string ClientApiKeyFile, bool doUpload) {
+/* TODO: Can't seem to get this to work
+std::unique_ptr<std::future<uint32_t>> HostCache::test_RestApiCall_async(const std::string api, const json j, const std::string ClientApiCertFile, const std::string ClientApiKeyFile, bool doUpload) {
+    std::future<uint32_t> f = std::async(RestApiCall, api, j, ClientApiCertFile, ClientApiKeyFile, doUpload, Debug);
+    std::unique_ptr<std::future<uint32_t>> fptr = std::make_unique<std::future<uint32_t>>(f);
+
+    return fptr;
+}
+*/
+
+void HostCache::RestApiCall_async (std::vector<std::future<uint32_t>> &futures, const std::string api, const json j, const std::string ClientApiCertFile, const std::string ClientApiKeyFile, bool doUpload) {
+     futures.emplace_back(std::async(RestApiCall, api, j, ClientApiCertFile, ClientApiKeyFile, doUpload, Debug));
+    // async(launch::async, (RestApiCall(api, j, ClientApiCertFile, ClientApiKeyFile, doUpload)).detach();
+}
+uint32_t RestApiCall (const std::string api, const json &j, const std::string ClientApiCertFile, const std::string ClientApiKeyFile, bool doUpload, bool Debug) {
 	std::string url = "https://api.noddos.io/" + api;
 
 	std::string body = j.dump();
@@ -668,7 +745,7 @@ uint32_t HostCache::RestApiCall (const std::string api, const json &j, const std
     return (uint32_t) response_code;
 }
 
-uint32_t HostCache::UploadDeviceStats(const std::string ClientApiCertFile, const std::string ClientApiKeyFile, bool doUpload) {
+void HostCache::UploadDeviceStats(std::vector<std::future<uint32_t>> &futures, const std::string ClientApiCertFile, const std::string ClientApiKeyFile, bool doUpload) {
 	uint32_t uploads = 0;
 	json j;
 	for (auto it : hC) {
@@ -681,33 +758,32 @@ uint32_t HostCache::UploadDeviceStats(const std::string ClientApiCertFile, const
 		}
 	}
 	if (uploads > 0) {
-		auto r = RestApiCall ("v1/uploaddevices", j, ClientApiCertFile, ClientApiKeyFile, doUpload);
-		syslog(LOG_INFO, "HostCache: Called v1/uploaddevices API with status_code %u", r);
+	    RestApiCall_async(futures, "v1/uploaddevices", j, ClientApiCertFile, ClientApiKeyFile, doUpload);
+		syslog(LOG_INFO, "HostCache: Called v1/uploaddevices API with for %u devices", uploads);
 	} else {
 		syslog(LOG_INFO, "HostCache: Not calling v1/uploaddevices API as there is no data to report");
 	}
-	return uploads;
+
 }
 
-bool HostCache::UploadTrafficStats(const time_t interval, const bool ReportRfc1918, const std::string ClientCertFile, const std::string ClientApiKeyFile, bool doUpload) {
+void HostCache::UploadTrafficStats(std::vector<std::future<uint32_t>> &futures, const time_t interval, const bool ReportRfc1918, const std::string ClientCertFile, const std::string ClientApiKeyFile, bool doUpload) {
 	uint32_t uploads = 0;
 	json j;
 	for (auto it : hC) {
 		if ( (not isWhitelisted(*(it.second))) && it.second->isMatched()) {
 			json h;
-			if (it.second->TrafficStats(h, interval, ReportRfc1918, LocalIpAddresses, dCip, false)) {
+			if (it.second->TrafficStats(h, interval, ReportRfc1918, LocalIpAddresses, dCipv4, dCipv6, false)) {
 				uploads++;
 				j.push_back(h);
 			}
 		}
 	}
 	if (uploads > 0) {
-		auto r = RestApiCall ("v1/uploadstats", j, ClientCertFile, ClientApiKeyFile, doUpload);
-		syslog(LOG_INFO, "HostCache: Called v1/uploadstats API with status_code %u", r);
+	    RestApiCall_async(futures, "v1/uploadstats", j, ClientCertFile, ClientApiKeyFile, doUpload);
+		syslog(LOG_INFO, "HostCache: Called v1/uploadstats API with for %u hosts", uploads);
 	} else {
 		syslog(LOG_INFO, "HostCache: Not calling v1/uploadstats API as there is no data to report");
 	}
-	return uploads;
 }
 
 uint32_t HostCache::ImportDeviceProfileMatches(const std::string filename) {
@@ -741,37 +817,74 @@ uint32_t HostCache::ImportDeviceProfileMatches(const std::string filename) {
 }
 
 void HostCache::updateDeviceProfileMatchesDnsData () {
+    if (Debug) {
+        syslog (LOG_DEBUG, "HostCache: starting updateDeviceProfileMatchesDnsData");
+    }
     for (auto dp_it: dpMap) {
         // Now we need to update the ipset rules for Device Profiles with
         // hasAllowedEndpoints and one or more hosts matched to it
+        if (Debug) {
+            syslog (LOG_DEBUG, "HostCache: updateDeviceProfileMatchesDnsData processing DP %s", dp_it.second->getDeviceDescription().c_str());
+        }
         dp_it.second->createorupdateIpsets();
         std::set<std::string> fqdns = dp_it.second->getDestinations();
         for (auto fqdn: fqdns) {
             fdpMap[fqdn].insert(dp_it.second);
+            if (Debug) {
+                syslog (LOG_DEBUG, "HostCache: updateDeviceProfileMatchesDnsData adding FQDN %s", fqdn.c_str());
+            }
             try {
-                std::map<boost::asio::ip::address, time_t> p = dCip.lookupResourceRecord(fqdn);
+                std::map<Tins::IPv4Address, time_t> p = dCipv4.lookupResourceRecord(fqdn);
                 for (auto ip_it: p) {
                     dp_it.second->addDestination(ip_it.first, ip_it.second);
                 }
             } catch (std::runtime_error &e) {
+                // No A records for FQDN
             }
-            std::string cname = fqdn;
             try {
-                while (1) {
-                    cname = dCcname.lookupCname(cname);
-                    try {
-                        std::map<boost::asio::ip::address, time_t> p = dCip.lookupResourceRecord(cname);
-                        for (auto ip_it: p) {
-                            dp_it.second->addDestination(ip_it.first, ip_it.second);
-                        }
-                    } catch (std::runtime_error &e) {
-                    }
-                    // All traffic allowed to a FQDN is also allowed to its CNAME
-                    // so if subsequently an A record is received for the CNAME,
-                    // the ipsets for the device profiles must be updated
-                    fdpMap[cname].insert(dp_it.second);
+                std::map<Tins::IPv6Address, time_t> p = dCipv6.lookupResourceRecord(fqdn);
+                for (auto ip_it: p) {
+                    dp_it.second->addDestination(ip_it.first, ip_it.second);
                 }
-            } catch (std::runtime_error &error) {}
+            } catch (std::runtime_error &e) {
+                // No AAAA records for FQDN
+            }
+            std::string cname = "";
+            while (fqdn != cname) {
+                try {
+                    cname = dCcname.getCname(fqdn);
+                } catch (std::runtime_error &e) {
+                    break;
+                }
+                if (Debug) {
+                    syslog (LOG_DEBUG, "HostCache: updateDeviceProfileMatchesDnsData adding CNAME %s for FQDN %s", cname.c_str(), fqdn.c_str());
+                }
+                try {
+                    std::map<Tins::IPv4Address, time_t> p = dCipv4.lookupResourceRecord(cname.c_str());
+                    for (auto ip_it: p) {
+                        dp_it.second->addDestination(ip_it.first, ip_it.second);
+                    }
+                } catch (std::runtime_error &e) {
+                    // No A records for FQDN
+                }
+                try {
+                    std::map<Tins::IPv6Address, time_t> p = dCipv6.lookupResourceRecord(cname.c_str());
+                    for (auto ip_it: p) {
+                        dp_it.second->addDestination(ip_it.first, ip_it.second);
+                    }
+                } catch (std::runtime_error &e) {
+                    // No A records for FQDN
+                }
+                // All traffic allowed to a FQDN is also allowed to its CNAME
+                // so if subsequently an A record is received for the CNAME,
+                // the ipsets for the device profiles must be updated
+                fdpMap[cname].insert(dp_it.second);
+                if (Debug) {
+                    syslog (LOG_DEBUG, "HostCache: updateDeviceProfileMatchesDnsData Finishing with FQDN %s and CNAME %s", fqdn.c_str(), cname.c_str());
+                }
+                // On nex iteration, do a CNAME lookup for the cname we found in this iteration
+                fqdn = cname;
+            }
         }
     }
 }
@@ -782,7 +895,8 @@ bool HostCache::exportDnsCache (const std::string filename) {
         return true;
     }
     json j;
-    dCip.exportJson(j);
+    dCipv4.exportJson(j);
+    dCipv6.exportJson(j);
     dCcname.exportJson(j);
 
     ofs << std::setw(4) << j << std::endl;
@@ -807,7 +921,8 @@ bool HostCache::importDnsCache (const std::string filename) {
     }
 
     try {
-        size_t dnsRecords = dCip.importJson(j, fdpMap);
+        size_t dnsRecords = dCipv4.importJson(j, fdpMap);
+        dnsRecords += dCipv6.importJson(j, fdpMap);
         if (Debug == true) {
             syslog(LOG_DEBUG, "HostCache: Read %zu cached DNS IP address records", dnsRecords);
         }
@@ -822,10 +937,15 @@ bool HostCache::importDnsCache (const std::string filename) {
     return false;
 }
 
-void HostCache::addorupdateDnsIpCache(const std::string inFqdn, const boost::asio::ip::address inIp, time_t inTtl) {
-    dCip.addorupdateResourceRecord(inFqdn, inIp, fdpMap, inTtl);
+void HostCache::addorupdateDnsIpCache(const std::string inFqdn, const Tins::IPv4Address inIp, time_t inTtl) {
+    dCipv4.addorupdateResourceRecord(inFqdn, inIp, fdpMap, inTtl);
 }
 
+void HostCache::addorupdateDnsIpCache(const std::string inFqdn, const Tins::IPv6Address inIp, time_t inTtl) {
+    dCipv6.addorupdateResourceRecord(inFqdn, inIp, fdpMap, inTtl);
+}
+
+
 void HostCache::addorupdateDnsCnameCache(const std::string inFqdn, const std::string inCname, time_t inTtl) {
     dCcname.addorupdateCname(inFqdn, inCname, fdpMap, inTtl);
 }
@@ -844,7 +964,7 @@ bool HostCache::removeDeviceProfile(const std::string inUuid) {
                 if (fdpmap_it != fdpMap.end()) {
                     fdpmap_it->second.erase(dp_it->second);
                 }
-                cname = dCcname.lookupCname(cname);
+                cname = dCcname.getCname(cname);
             }
         } catch (std::runtime_error &error) {}
     }
@@ -899,15 +1019,19 @@ bool HostCache::ImportDeviceInfo (json &j) {
 	MacAddress Mac(MacAddressString);
 	auto hit = hC.find(Mac.get());
 	if (hit != hC.end()) {
-		std::string uuid = hit->second->Uuid_get();
+		std::string uuid = hit->second->getUuid();
 		if (uuid != DeviceProfileUuid) {
 			syslog(LOG_WARNING, "HostCache: Conflicting Uuid for imported device with existing Host Cache");
 			return false;
 		}
 	}
-	if (not FindOrCreateHostByMac(Mac, DeviceProfileUuid, Ipv4Address)) {
-		syslog(LOG_WARNING, "HostCache: Failed to create Host with MacAddress %s and uuid %s", MacAddressString.c_str(), DeviceProfileUuid.c_str());
-		return false;
+	try {
+	    if (not FindOrCreateHostByMac(Mac, DeviceProfileUuid, Ipv4Address)) {
+	        syslog(LOG_WARNING, "HostCache: Failed to create Host with MacAddress %s and uuid %s", MacAddressString.c_str(), DeviceProfileUuid.c_str());
+	        return false;
+	    }
+	} catch (...) {
+	    return false;
 	}
 
 	try {
@@ -986,12 +1110,18 @@ uint32_t HostCache::Whitelists_set (const std::unordered_set<std::string>& inIpv
 }
 
 void HostCache::writeIptables()  {
+    if (FirewallRulesFile == "") {
+        if (Debug == true) {
+            syslog(LOG_DEBUG, "Iptables: Not writing firewall rules as feature is disabled");
+        }
+        return;
+    }
     if (Debug == true) {
         syslog(LOG_DEBUG, "Iptables: Writing firewall rules to %s", FirewallRulesFile.c_str());
     }
     std::ofstream outputfs(FirewallRulesFile);
     std::vector<std::string> ifaces = ifMap->getLanInterfaces();
-    std::string action = "LOG";
+    std::string action = "LOG --log-prefix Noddos-IPtables-LOG ";
     if (FirewallBlockTraffic == true) {
         action = "DROP";
     }
@@ -999,7 +1129,15 @@ void HostCache::writeIptables()  {
     outputfs << "*filter" << std::endl;
 
     for (auto dp_it: dpMap) {
+        if (Debug == true) {
+            syslog(LOG_DEBUG, "Iptables: Processing Device Profile %s with endpoints? %sand with hosts associated? %s",
+                    dp_it.second->getUuid().c_str(), dp_it.second->hasAllowedEndpoints() ? "true" : "false",
+                    dp_it.second->hasHosts() ? "true" : "false");
+        }
         if (dp_it.second->hasAllowedEndpoints() && dp_it.second->hasHosts()) {
+            if (Debug == true) {
+                syslog(LOG_DEBUG, "Iptables: Device Profile has endpoints");
+            }
             std::string srcipset = getIpsetName(dp_it.second->getUuid(),true,false);
             std::string ipv46flag;
             std::string dstipset;
@@ -1009,18 +1147,20 @@ void HostCache::writeIptables()  {
                 ipv46flag = "--ipv4";
                 outputfs << "-A NODDOS -i " + iface + " " + ipv46flag +
                         " -m set --match-set " + srcipset + " src " +
-                        "-m set --match-set " +  dstipset + " dst -j ACCEPT" << std::endl;
-
+                        "-m set --match-set " +  dstipset + " dst -j ACCEPT"
+                        " -m comment --comment \"" + dp_it.second->getDeviceDescription() + "\"" << std::endl;
                 // IPv6 permit rule
                 dstipset = getIpsetName(dp_it.second->getUuid(), false, false);
                 ipv46flag = "--ipv6";
                 outputfs << "-A NODDOS -i " + iface + " " + ipv46flag +
                         " -m set --match-set " + srcipset + " src " +
-                        "-m set --match-set " +  dstipset + " dst -j ACCEPT" << std::endl;
+                        "-m set --match-set " +  dstipset + " dst -j ACCEPT"
+                        " -m comment --comment \"" + dp_it.second->getDeviceDescription() + "\"" << std::endl;
 
                 // Block all other traffic from the MAC addresses mapped to the Device Profile
                 outputfs << "-A NODDOS -i " + iface +
-                        " -m set --match-set " + srcipset + " src -j " + action << std::endl;
+                        " -m set --match-set " + srcipset + " src -j " + action <<
+                        " -m comment --comment \"" + dp_it.second->getDeviceDescription() + "\"" << std::endl;
             }
         }
 
