@@ -17,18 +17,20 @@
  * noddos.cxx
  *
  *  Created on: Mar 11, 2017
- *      Author: steven
+ *      Author: Steven Hessing (steven.hessing@gmail.com)
  */
 #include <map>
 #include <string>
 #include <system_error>
-
-#include <ctime>
-#include <cstring>
+#include <future>
 #include <fstream>
 #include <iostream>
 #include <memory>
 #include <unordered_set>
+#include <vector>
+#include <chrono>
+#include <ctime>
+#include <cstring>
 #include <csignal>
 #include <sys/epoll.h>
 #include <stdlib.h>
@@ -53,6 +55,8 @@
 #include "DeviceProfile.h"
 #include "Host.h"
 #include "PacketSnoop.h"
+#include "WsDiscovery.h"
+#include "Mdns.h"
 #include "InterfaceMap.h"
 #include "Config.h"
 #include "noddos.h"
@@ -93,17 +97,22 @@ int main(int argc, char** argv) {
 	}
 	write_pidfile(config.PidFile);
 
+	//
+	// Set up CURL initializer
+	//
 	CURLcode cc = curl_global_init(CURL_GLOBAL_ALL);
 	if (cc != 0) {
 	    syslog (LOG_CRIT, "Noddos: Curl init failure: %d", cc);
 	}
 
+	// Vector containing threads for HTTPS API calls to Noddos cloud
+	std::vector<std::future<uint32_t>> futures;
 
 	//
 	// Set up HostCache instance
 	//
 	HostCache hC(ifMap, config.DnsCacheFile, config.TrafficReportInterval,
-	        config.FirewallRulesFile, config.FirewallBlockTraffic, config.Debug);
+	        config.FirewallRulesFile, config.FirewallBlockTraffic, config.Debug && config.DebugHostCache);
 	hC.loadDeviceProfiles(config.DeviceProfilesFile);
 	hC.ImportDeviceProfileMatches(config.MatchFile);
 	hC.Whitelists_set(config.WhitelistedIpv4Addresses, config.WhitelistedIpv6Addresses, config.WhitelistedMacAddresses);
@@ -127,7 +136,9 @@ int main(int argc, char** argv) {
     	syslog(LOG_ERR, "Noddos: Setting up signal fd");
     	throw std::system_error(errno, std::system_category());
     } else {
-    	syslog(LOG_INFO, "Noddos: Signal FD is: %d", sfd);
+    	if (config.Debug == true) {
+    	    syslog(LOG_DEBUG, "Noddos: Signal FD is: %d", sfd);
+    	}
         struct epoll_event event;
         memset (&event, 0, sizeof (event));
         event.data.fd = sfd;
@@ -149,28 +160,59 @@ int main(int argc, char** argv) {
     std::unordered_set<std::string> allInterfaces = config.LanInterfaces;
     allInterfaces.insert(config.WanInterfaces.begin(), config.WanInterfaces.end());
     for (auto iface: allInterfaces) {
-        PacketSnoop *p = new PacketSnoop(hC, 64, config.Debug);
+        PacketSnoop *p = new PacketSnoop(hC, 64, config.Debug && config.DebugPacketSnoop);
         p->Open(iface, 64);
         add_epoll_filehandle(epfd, epollmap, *p);
+        if (config.Debug) {
+            syslog(LOG_DEBUG, "Noddos: PacketSnoop for interface %s file handle %d", iface.c_str(), p->getFileHandle());
+        }
         pInstances.insert(p);
     }
 
-    SsdpServer s(hC, 86400, "", config.Debug);
+    SsdpServer s(hC, 86400, "", config.Debug && config.DebugSsdp);
     add_epoll_filehandle(epfd, epollmap, s);
+    if (config.Debug) {
+        syslog(LOG_DEBUG, "Noddos: SSDP file handle %d", s.getFileHandle());
+    }
 
-    FlowTrack ft(hC, config) ;
+    WsDiscovery w(hC, 86400, "", config.Debug && config.DebugWsDiscovery);
+    add_epoll_filehandle(epfd, epollmap, w);
+    if (config.Debug) {
+        syslog(LOG_DEBUG, "Noddos: WS-Discovery file handle %d", w.getFileHandle());
+    }
+
+    Mdns m(hC, 86400, "", config.Debug && config.DebugMdns);
+    add_epoll_filehandle(epfd, epollmap, m);
+    if (config.Debug) {
+        syslog(LOG_DEBUG, "Noddos: mDNS file handle %d", m.getFileHandle());
+    }
+
+    std::set<std::string> localIpAddresses = hC.getLocalIpAddresses();
+    FlowTrack ft(hC, config, localIpAddresses) ;
     ft.Open();
    	add_epoll_filehandle(epfd, epollmap, ft);
+    if (config.Debug) {
+        syslog(LOG_DEBUG, "Noddos: FlowTrack file handle %d", ft.getFileHandle());
+    }
 
     if (config.User != "" && config.Group != "") {
     	drop_process_privileges(config);
     }
     uint32_t NextMatch = time(nullptr) + config.MatchInterval + rand() % 15;
     uint32_t NextPrune = time(nullptr) + config.PruneInterval + rand() % 15;
-	uint32_t NextDeviceUpload = time(nullptr) + config.DeviceReportInterval + rand() %5;
-	uint32_t NextTrafficUpload = time(nullptr) + config.TrafficReportInterval + rand() %5;
+	uint32_t NextDeviceUpload = time(nullptr) + config.DeviceReportInterval + rand() % 5;
+	uint32_t NextTrafficUpload = time(nullptr) + config.TrafficReportInterval + rand() % 5;
+	uint32_t NextWsDiscoveryProbe = 0; // Send out probe at first opportunity
 
 	struct epoll_event* epoll_events = static_cast<epoll_event*>(calloc(MAXEPOLLEVENTS, sizeof (epoll_event)));
+
+	//
+	// epoll setup complete
+	//
+
+	//
+	// main program loop waiting for events
+	//
 	while (true) {
     	if (config.Debug) {
     		syslog(LOG_DEBUG, "Noddos: Starting epoll event wait");
@@ -185,12 +227,16 @@ int main(int argc, char** argv) {
 		int ev;
     	for (ev = 0; ev< eCnt; ev++) {
 
-			if ((epoll_events[ev].events & EPOLLERR) || (epoll_events[ev].events & EPOLLHUP) ||
+			//
+    	    // epoll event error handling
+    	    //
+    	    if ((epoll_events[ev].events & EPOLLERR) || (epoll_events[ev].events & EPOLLHUP) ||
                     (not epoll_events[ev].events & EPOLLIN)) {
 				syslog(LOG_ERR, "Noddos: Epoll event error for FD %d", epoll_events[ev].data.fd);
 				epollmap.erase(epoll_events[ev].data.fd);
 				if (epoll_events[ev].data.fd == ft.getFileHandle() && geteuid() == 0) {
-					ft.Close();
+					// Connection tracking has shown it can have hickups so we re-initiate it when that happens
+				    ft.Close();
 					ft.Open();
 			    	add_epoll_filehandle(epfd, epollmap, ft);
 				} else {
@@ -198,11 +244,16 @@ int main(int argc, char** argv) {
 				    close(epoll_events[ev].data.fd);
 				}
 			} else {
+			    //
+			    // epoll event processing
+			    //
 				if (config.Debug) {
 					syslog(LOG_DEBUG, "Noddos: Handling event for FD %d", epoll_events[ev].data.fd);
 				}
 				if (epoll_events[ev].data.fd == sfd) {
-					// Signal received
+					//
+				    // Signal processing
+				    //
 					if (config.Debug) {
 						syslog(LOG_DEBUG, "Processing signal event");
 					}
@@ -231,8 +282,8 @@ int main(int argc, char** argv) {
 					} else if (si.ssi_signo == SIGUSR2) {
 						syslog(LOG_INFO, "Noddos: Processing signal event SIGUSR2");
 						hC.Match();
-						hC.UploadDeviceStats(config.ClientApiCertFile, config.ClientApiKeyFile, config.DeviceReportInterval != 0);
-						hC.UploadTrafficStats(config.TrafficReportInterval, config.ReportTrafficToRfc1918,
+						hC.UploadDeviceStats(futures, config.ClientApiCertFile, config.ClientApiKeyFile, config.DeviceReportInterval != 0);
+						hC.UploadTrafficStats(futures, config.TrafficReportInterval, config.ReportTrafficToRfc1918,
 								config.ClientApiCertFile, config.ClientApiKeyFile, config.TrafficReportInterval != 0);
 						NextDeviceUpload = time(nullptr) + config.DeviceReportInterval;
 						NextTrafficUpload = time(nullptr) + config.TrafficReportInterval;
@@ -241,9 +292,14 @@ int main(int argc, char** argv) {
 					}
 					setup_signal_fd(sfd);
 				} else {
+				    //
+				    // event for on of the iDeviceInfoSource child classes
+				    //
 					iDeviceInfoSource &i = *(epollmap[epoll_events[ev].data.fd]);
 					i.processEvent(epoll_events[ev]);
 				}
+				//
+				// Timer handling
 				auto t = time(nullptr);
 				if (t > NextMatch) {
 					hC.Match();
@@ -252,14 +308,14 @@ int main(int argc, char** argv) {
 					hC.ExportDeviceProfileMatches(config.DumpFile, true);
 				}
 				if (t > NextDeviceUpload && config.DeviceReportInterval > 0) {
-					hC.UploadDeviceStats(config.ClientApiCertFile, config.ClientApiKeyFile, config.DeviceReportInterval != 0);
+					hC.UploadDeviceStats(futures, config.ClientApiCertFile, config.ClientApiKeyFile, config.DeviceReportInterval != 0);
 					NextDeviceUpload = t + config.DeviceReportInterval;
 				}
 				if (t > NextTrafficUpload && config.TrafficReportInterval > 0) {
-					if (config.Debug) {
+				    if (config.Debug) {
 						syslog(LOG_DEBUG, "Noddos: Starting traffic upload");
 					}
-					hC.UploadTrafficStats(config.TrafficReportInterval, config.ReportTrafficToRfc1918, config.ClientApiCertFile,
+					hC.UploadTrafficStats(futures, config.TrafficReportInterval, config.ReportTrafficToRfc1918, config.ClientApiCertFile,
 							config.ClientApiKeyFile, config.TrafficReportInterval != 0);
 					NextTrafficUpload = t + config.TrafficReportInterval;
 				}
@@ -273,7 +329,32 @@ int main(int argc, char** argv) {
 					}
 					NextPrune = t + config.PruneInterval;
 				}
+				if (t > NextWsDiscoveryProbe && config.WsDiscoveryProbeInterval > 0) {
+				    if (config.Debug == true) {
+				        syslog(LOG_DEBUG, "Noddos: Starting WS-Discovery Probe");
+				    }
+				    w.Probe();
+				    NextWsDiscoveryProbe = t + config.WsDiscoveryProbeInterval;
+				}
+
     		}
+    	    //
+    	    // Processing of completion of HTTPS API calls
+    	    //
+			if (futures.size() > 0) {
+			    for (auto future_it = futures.begin(); future_it != futures.end();) {
+			        if (future_it->valid()) {
+			            if (future_it->wait_for(std::chrono::seconds(0)) == std::future_status::ready) {
+		                    if (config.Debug == true) {
+		                        syslog(LOG_DEBUG, "Noddos: Upload of data returned HTTP status %u", future_it->get());
+		                    }
+			                future_it = futures.erase(future_it);
+			            } else {
+			                future_it++;
+			            }
+			        }
+			    }
+			}
     	}
 
     }
@@ -282,12 +363,14 @@ exitprog:
     hC.exportDnsCache(config.DnsCacheFile);
     hC.Prune();
 	s.Close();
+	w.Close();
 	for (auto p: pInstances) {
 	    p->Close();
 	}
 	ft.Close();
 	close (epfd);
 	close (sfd);
+	syslog (LOG_INFO, "Noddos: Exiting");
 	closelog();
 	unlink (config.PidFile.c_str());
 	free (epoll_events);
@@ -391,12 +474,16 @@ bool daemonize (Config &inConfig) {
 	if (ifs.is_open()) {
 		ifs >> origpid;
 		std::string pidprocpath = "/proc/" + origpid + "/stat";
-		fprintf (stderr, "Checking if pid file %s exists\n", pidprocpath.c_str());
+		if (inConfig.Debug == true) {
+		    fprintf (stderr, "Checking if pid file %s exists\n", pidprocpath.c_str());
+		}
 		struct stat buf;
 		if (stat (pidprocpath.c_str(), &buf) == 0) {
 		    throw std::runtime_error ("Pid file " + inConfig.PidFile + " exists and contains PID of a running process ");
 		}
-		fprintf (stderr, "Deleting stale pid file %s\n", pidprocpath.c_str());
+        if (inConfig.Debug == true) {
+            fprintf (stderr, "Deleting stale pid file %s\n", pidprocpath.c_str());
+        }
 		unlink(inConfig.PidFile.c_str());
 		ifs.close();
 	}
@@ -408,7 +495,7 @@ bool daemonize (Config &inConfig) {
 	pid = fork();
 	// The parent process continues with a process ID greater than 0
 	if(pid > 0)	{
-	    fprintf (stderr, "We've forked so existing parent process\n");
+	    // fprintf (stderr, "We've forked so existing parent process\n");
 	    exit(0);
 	}
 	// A process ID lower than 0 indicates a failure in either process
